using dg1d
using CairoMakie
CairoMakie.activate!()


function analyze(dir)

  tomlfile = get_parameter_file(dir)
  parameters = FlatDict(TOML.parsefile(joinpath(dir, tomlfile)))

  mesh = make_Mesh(trim_headers(parameters, "Mesh"))

  thisdir = dirname(@__FILE__)
  output = Output(joinpath(thisdir, first(splitext(tomlfile)), "output.h5"))
  ts = output.times

  # select samples to plot
  nsamples = 5
  ntotal   = length(ts)
  nstep    = trunc(Int64, ntotal / (nsamples-1))
  rng      = collect(1:nstep:ntotal)
  plot_ts     = ts[rng]
  plot_us     = [ read_variable(output, "dynamic_variables.u", t) for t in plot_ts ]
  
  # plotting
  fig = Figure(resolution=(1600,1200))
  colors = to_colormap(:rainbow, length(rng))
  # dim 'older' results
  transparency(idx, n) = exp((idx-n)*2/n)
  ax1 = Axis(fig[2,1], ylabel=L"u", xlabel=L"x")
  L = layout(mesh)
  Npts, K = L
  x_w_nans = make_nan_separated_storage(L)
  x_w_nans[1:end-1,:] .= mesh.x
  y_w_nans = make_nan_separated_storage(L)

  handles, labels = [], String[]
  for (i, (u, t)) in enumerate(zip(plot_us, plot_ts))
    mat_u     = reshape(u, L)
    transp = transparency(i, nsamples+1)
    y_w_nans[1:end-1,:] .= mat_u
    h = lines!(ax1, x_w_nans[:], y_w_nans[:], color=(colors[i],transp))
    push!(handles, h)
    push!(labels, "t=$t")
  end

  xlims!(ax1, extrema(mesh.x))
  # xlims!(ax2, extrema(mesh.x))

  Legend(fig[1,1], handles, labels, orientation=:horizontal)

  save(joinpath(dir, "evolution.png"), fig)

  return
end


#######################################################################
#                                Main                                 #
#######################################################################


projectdir = Filesystem.abspath(Filesystem.dirname(@__FILE__))
dg1d.analyze_projects(analyze, projectdir)
