#!/usr/bin/env python3


from sympy import *


def spprint(expr):
    pprint(simplify(expr))


def lagrange_polynomials(x, x_ref):
    n = len(x_ref)
    ls = []
    for i in range(n):
        xi = x_ref[i]
        l = 1
        for j in range(n):
            if i == j: 
                continue
            xj = x_ref[j]
            l *= (x - xj) / (xi - xj)
        ls.append(l)

    return ls

xs = [-1, 0, 1]
x = symbols('x')
ls = lagrange_polynomials(x, xs)

v0, v1, v2 = symbols('v:3')
vs = [v0, v1, v2]
v = 0
for (vi, li) in zip(vs, ls):
    v += vi * li

d, g = symbols('d g')
sign = -1
ansatz = d * (x + sign * g)**2

coeffs_ansatz   = ansatz.as_poly(x).all_coeffs()
coeffs_v        = v.as_poly(x).all_coeffs()

eqs = []
if len(coeffs_v) == len(coeffs_ansatz):
    for (c_a, c_v) in zip(coeffs_ansatz, coeffs_v):
        eqs.append(c_a - c_v)
else:
    print("Mismatch between ansaetze!")

#  for eq in eqs:
#      spprint(eq)

#  a, b = symbols('a b')
#  # a = v0 + v2
#  eqs[0] = eqs[0].subs(v0, a - v2)
#  # b = -v0 + v2
#  eqs[1] = eqs[1].subs(v2, b + v0)

vv = symbols('vv')
for (i,eq) in enumerate(eqs):
    eqs[i] = eq.subs(v0,vv).subs(v2,vv)

#  for eq in eqs:
#      spprint(eq)

vars = [g, d, v1]
sol = nonlinsolve(eqs, vars)
for s in sol:
    spprint(s)

#  s = sol.args[0]
#  spprint(s)
#
#  g, d, v1 = s
#  pprint(g * sqrt(d))


def coeffs(sign_gamma):
    d, g, a, b = symbols('d g a b')

    eq1 = d * (1 + g**2) - a
    eq2 = sign_gamma * 2 * g * d - b

    sol = nonlinsolve([eq1, eq2], [d, g])

    for (i, s) in enumerate(sol):

        print(f"Solution {i}")

        di, gi = s
        print("delta = ")
        spprint(di)
        print("gamma = ")
        spprint(gi)

        print("Root: sqrt(delta) * gamma = ")
        root = sqrt(di) * gi
        spprint(root)

        print("Curvature: sqrt(delta) =")
        curv = sqrt(di)
        spprint(curv)

        print("v(extremum) = ")
        spprint(4 * di * gi**2)

        print("\n")

