using dg1d
using GLMakie
using CairoMakie
using Random
GLMakie.activate!()


function plot_bernstein_reconstruction()

  stepdata(x, x0) = [ (xi <= x0) ? -1.0 : 1.0 for xi in x ]

  x0 = 0.0

  Nref = 100
  xref = collect(LinRange(-1.0, 1.0, Nref))
  fref = stepdata(xref, x0)
  xref_w_nans, fref_w_nans =
    [ dg1d.make_nan_separated_storage((Nref,1)) for _ = 1:2 ]
  xref_w_nans[1:end-1,:]  .= xref
  fref_w_nans[1:end-1,:]  .= fref

  Ns         = [1, 2, 3, 4, 5, 6, 7, 8, 9, 19, 29 ]
  # Ns         = [2, 4, 6, 8]
  meshes     = [ make_Mesh(N=N,K=1) for N in Ns ]
  bernsteins = [ HRSC.BernsteinReconstruction(mesh) for mesh in meshes ]
  Is         = [ dg1d.barycentric_interp_matrix(xref, mesh.x) for mesh in meshes ]
  # Is         = [ dg1d.lagrange_interp_matrix(xref, mesh.x) for mesh in meshes ]

  fig = Figure(resolution=(1600,1200))
  for (i,w) in enumerate([0,1/2,1])

    Label(fig[1,i], "w = $w", tellwidth=false)
    ax = Axis(fig[2,i])
    lines!(ax, xref_w_nans[:], fref_w_nans[:])
    for (N, mesh, bernstein, I) in zip(Ns, meshes, bernsteins, Is)

      f = stepdata(mesh.x, x0)
      bf = copy(f)
      HRSC.reconstruct!(bf, [w], bernstein,
                        limit_with_boundaries=false #=adjust this to toggle overshoots=#)
      x_w_nans, bf_w_nans = [ dg1d.make_nan_separated_storage((Nref,1)) for _ = 1:2 ]
      bf_w_nans[1:end-1,:] .= I * bf

      scatterlines!(ax, xref_w_nans[:], bf_w_nans[:], label="N = $N", markersize=5)
    end
    axislegend(ax, position=:lt)

  end

  return fig
end


# CairoMakie.activate!()
# save(joinpath(@__DIR__, "even_lagranges_fixed_left.pdf"), fig)
# GLMakie.activate!()
