export numericalflux_lax_friedrich_local!,
       numericalflux_central!,
       numericalflux_strongform!


function numericalflux_lax_friedrich_local!(nf_lhs, nf_rhs, f, u, local_v, mesh::Mesh)
  shape = layout(mesh)
  _, K = shape
  Idx = LinearIndices(shape)
  # domain bdry interfaces are assumed to be periodically connected
  for k = 1:K # interface index
    k_lhs, k_rhs = periodic_index(k-1, K), periodic_index(k, K)
    # values **to** the lhs/rhs of kth interface
    fbdry_lhs, fbdry_rhs = f[Idx[end,k_lhs]], f[Idx[1,k_rhs]]
    ubdry_lhs, ubdry_rhs = u[Idx[end,k_lhs]], u[Idx[1,k_rhs]]
    v = max(abs(local_v[k_lhs]), abs(local_v[k_rhs])) # take maximum in surrounding cells
    # Convention:
    # nf_lhs[k] corresponds to the numerical flux at the lhs end of the kth cell
    # Sketch
    # numerical flux:           nf_lhs[k]  nf_rhs[k]
    # cell idx:               k-1   ↓    k    ↓    k+1
    #                         ↓          ↓         ↓
    # |---------|---------|---------|---------|---------|---------|---------|
    #                       (k_lhs)   (k_rhs)
    #                     ↑         ↑         ↑
    # interface idx:     k-1        k        k+1
    #
    avg_fbdry = 0.5 * (fbdry_lhs + fbdry_rhs)
    dif_ubdry = ubdry_lhs - ubdry_rhs
    nf_lhs[k_rhs] = avg_fbdry + 0.5 * v * dif_ubdry
    nf_rhs[k_lhs] = avg_fbdry + 0.5 * v * dif_ubdry
  end
  return
end


function numericalflux_central!(nf_lhs, nf_rhs, u, mesh::Mesh)
  shape = layout(mesh)
  Idx = LinearIndices(shape)
  _, K = shape
  # domain bdry interfaces are assumed to be periodically connected
  for k = 1:K # interface index
    k_lhs, k_rhs = periodic_index(k-1, K), periodic_index(k, K)
    # values to the lhs/rhs of kth interface
    ubdry_lhs, ubdry_rhs = u[Idx[end,k_lhs]], u[Idx[1,k_rhs]]
    avg_ubdry = 0.5 * (ubdry_lhs + ubdry_rhs)
    # for index convention see numericalflux_lax_friedrich_local!
    nf_lhs[k_rhs] = avg_ubdry
    nf_rhs[k_lhs] = avg_ubdry
  end
  return
end


function numericalflux_strongform!(nf_lhs, nf_rhs, f, mesh::Mesh)
  shape = layout(mesh)
  Idx = LinearIndices(shape)
  _, K = shape
  # domain bdry interfaces are assumed to be periodically connected
  for k = 1:K # interface index
    k_lhs, k_rhs = periodic_index(k-1, K), periodic_index(k, K)
    # values to the lhs/rhs of kth interface
    fbdry_lhs, fbdry_rhs = f[Idx[end,k_lhs]], f[Idx[1,k_rhs]]
    # for index convention see numericalflux_lax_friedrich_local!
    nf_lhs[k_rhs] = fbdry_rhs - nf_lhs[k_rhs]
    nf_rhs[k_lhs] = fbdry_lhs - nf_rhs[k_lhs]
  end
  return
end
