export AbstractParameters,
       sanitize


#######################################################################
#                                Types                                #
#######################################################################


@doc """
  abstract AbstractParameters

Supertype for participating in the Parameters interface.

Subtyped structs can implement the following methods to verify that
provided parameters are sane:
```
sanitize(prms::T) where T<:AbstractParameters
```
"""
abstract type AbstractParameters end


#######################################################################
#                          Interface methods                          #
#######################################################################


function sanitize(prms::AbstractParameters) end


#######################################################################
#                               Methods                               #
#######################################################################


function Base.show(io::IO, prms::AbstractParameters)
  if Configurations.is_option(prms)
    print(strip(to_toml(prms)))
  else
    print(prms)
  end
end


function Base.show(io::IO, ::MIME"text/plain", prms::AbstractParameters)
  show(io, prms)
end
