function make_BoundaryConditions(rhs::RHS, prms)
  type = query(String, prms, "type")

  lhs_bc, rhs_bc = @match type begin
    "periodic"     => (PeriodicBC(), PeriodicBC())
    "from initial data" => begin
      @unpack rho, q, E = get_dynamic_variables(rhs.cache)
      lhs_state = (rho[1], q[1], E[1])
      rhs_state = (rho[end], q[end], E[end])
      (DirichletBC(lhs_state), DirichletBC(rhs_state))
    end
    _ => error("Boundary condition type '$type' not implemented!")
  end

  return BoundaryConditions(lhs_bc, rhs_bc, rhs.rsolver)
end


function make_BoundaryConditions_LDG(ldg_rsolver, prms)
  isnothing(ldg_rsolver) && return nothing
  type = query(String, prms, "type")
  lhs_bc, rhs_bc = if type == "periodic"
    PeriodicBC(), PeriodicBC()
  else
    OutflowBC(), OutflowBC()
  end
  return BoundaryConditions(lhs_bc, rhs_bc, ldg_rsolver)
end
