"""
    AbstractArtificialViscosity


Supertype for Artificial Viscosity (AV) interface.

Artificial viscosity aims to resolve shocks and discontinuities
by regularizing a first order hyperbolic conservation law with a second
order spatial derivative term, e.g.
  
  ∂t u + ∂x f(u) = 0   =>   ∂t u + ∂x f(u) = ∂x(ν(x) ∂x u)
"""
abstract type AbstractArtificialViscosity <: AbstractHRSC end


"""
    timestep(mumax, vmax, eq::AbstractEquation, av::ArtificialViscosity)

Compute artificial viscosity `mu` inplace based on current solution `u`, local speeds `v`,
time `t`.
"""
function timestep(vmax, mumax, mesh::Mesh, av::AbstractArtificialViscosity)
  dt = CFL / (vmax * N^2 / dl + mumax * N^4 / dl^2)
  return dt
end


# viscosity models
include("mda.jl")
include("mdh.jl")
include("db.jl")
include("ev.jl")
include("smoother.jl")
include("tciav.jl")
