@doc """
    TCIViscosity

Generic Artificial Viscosity method based on any troulbed cell indicator.
"""
Base.@kwdef struct TCIViscosity <: AbstractArtificialViscosity
  mesh::Mesh
  mu_threshold::Float64
  mu_max::Float64
end


function parse_parameters(::Type{TCIViscosity}, prms)
  mu_threshold = query(Float64, prms, "mu_threshold", default=1e-5)
  mu_max       = query(Float64, prms, "mu_max", default=1e-2)
  @assert mu_max > mu_threshold
  return (; mu_threshold, mu_max)
end


function compute_viscosity!(
    mu, 
    flag,
    av::TCIViscosity)

  @unpack mesh, mu_threshold, mu_max = av
  Npts, K                            = layout(mesh)

  for k = 1:K
    muk = flag[k] * mu_max
    if muk < mu_threshold
      muk = 0
    end
    mu[k] = muk
  end

  return true
end
