Base.@kwdef struct Diffusion <: AbstractTCI
  mesh::Mesh
  # parameters
  max_diffusion::Float64
end


function parse_parameters(::Type{Diffusion}, prms)
  max_diffusion = query(Float64, prms, "max_diffusion", default = 1.0)
  @assert max_diffusion > 0
  return (; max_diffusion)
end


function compute_indicator!(
    flag, # mutated outputs
    u,    # inputs
    tci::Diffusion)

  @unpack mesh   = tci
  @unpack invjac = mesh
  @unpack D      = mesh.element
  L              = layout(mesh)
  Npts, K        = L

  u           = reshape(u, L)
  grad_u_cell = zeros(Npts)

  @unpack max_diffusion = tci
  for k = 1:K
    grad_u_cell .= D * u[:,k] .* invjac[k]
    max_grad_u = maximum(abs.(grad_u_cell))
    max_grad_u = minimum((max_grad_u, max_diffusion))
    flag[k] = max_grad_u / max_diffusion
  end

  return
end
